require File.join(File.dirname(__FILE__), "..", "helpers.rb")

require "sensu/server/process"

describe "Sensu::Server::Filter" do
  include Helpers

  before do
    @server = Sensu::Server::Process.new(options)
    @handler = {}
    @event = event_template
  end

  it "can determine if handler handles a silenced event" do
    expect(@server.handler_silenced?(@handler, @event)).to be(false)
    @event[:silenced] = true
    expect(@server.handler_silenced?(@handler, @event)).to be(true)
    @handler[:handle_silenced] = true
    expect(@server.handler_silenced?(@handler, @event)).to be(false)
    @handler[:handle_silenced] = false
    expect(@server.handler_silenced?(@handler, @event)).to be(true)
  end

  it "can determine if handling is disabled for an event" do
    expect(@server.handling_disabled?(@event)).to be(false)
    @event[:check][:handle] = false
    expect(@server.handling_disabled?(@event)).to be(true)
  end

  it "can determine if a handler handles an action" do
    expect(@server.handle_action?(@handler, @event)).to be(true)
    @event[:action] = :flapping
    expect(@server.handle_action?(@handler, @event)).to be(false)
    @handler[:handle_flapping] = true
    expect(@server.handle_action?(@handler, @event)).to be(true)
  end

  it "can determine if a handler handles a severity" do
    handler = {
      :severities => ["critical"]
    }
    expect(@server.handle_severity?(handler, @event)).to be(false)
    @event[:check][:status] = 2
    expect(@server.handle_severity?(handler, @event)).to be(true)
    @event[:check][:status] = 0
    expect(@server.handle_severity?(handler, @event)).to be(false)
    @event[:action] = :resolve
    @event[:check][:history] = [1, 0]
    expect(@server.handle_severity?(handler, @event)).to be(false)
    @event[:check][:history] = [1, 2, 0]
    expect(@server.handle_severity?(handler, @event)).to be(true)
  end

  it "can determine if filter attributes match an event" do
    attributes = {
      :occurrences => 1,
      :action => "resolve"
    }
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
    attributes[:action] = "create"
    expect(@server.filter_attributes_match?(@event, attributes)).to be(true)
    @event[:occurrences] = 2
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
  end

  it "can determine if filter eval attributes match an event" do
    attributes = {
      :occurrences => "eval: value == 3 || value % 60 == 0"
    }
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
    @event[:occurrences] = 3
    expect(@server.filter_attributes_match?(@event, attributes)).to be(true)
    @event[:occurrences] = 4
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
    @event[:occurrences] = 120
    expect(@server.filter_attributes_match?(@event, attributes)).to be(true)
    @event[:occurrences] = 3
    attributes[:occurrences] = "eval: value == :::check.occurrences:::"
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
    attributes[:occurrences] = "eval: value == :::check.occurrences|3:::"
    expect(@server.filter_attributes_match?(@event, attributes)).to be(true)
    @event[:check][:occurrences] = 2
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
    @event[:occurrences] = 2
    expect(@server.filter_attributes_match?(@event, attributes)).to be(true)
  end

  it "can catch SyntaxErrors in eval filters" do
    attributes = {
      :occurrences => "eval: raise SyntaxError"
    }
    expect(@server.filter_attributes_match?(@event, attributes)).to be(false)
  end
end
